@@ -78,7 +78,7 @@ class MCalc_White_et_al_2015(MetricCalculator):
             outArray[3] = scipy.stats.kurtosis(points[:, 2])
             outArray[4:6] = np.percentile(points[:, 2], [10, 90])
             outArray[6] = np.count_nonzero(points[:, 2] > outArray[0]) / points.shape[0]
-            rumple = rumple_index(points, rumple_pixel_size)
+            rumple = rumple_index(points_in_poly, rumple_pixel_size)
             outArray[7] = rumple
 
         return outArray
